@@ -42,6 +42,7 @@ COMMON = [
     'ccache',
     'curl',
     'chrpath',
+    'codespell',
     'flex',
     'gcc',
     'gdb',
@@ -49,6 +50,7 @@ COMMON = [
     'gzip',
     'hostname',
     'htop',
+    'jq',
     'lcov',
     'make',
     'patch',
@@ -57,10 +59,12 @@ COMMON = [
     'rng-tools',
     'rsync',
     'sed',
+    'shfmt',
     'sudo',  # docker images has no sudo by default
     'tar',
     'tree',
     'wget',
+    'cargo',
 ]
 
 
@@ -80,6 +84,9 @@ PKGS = [
     ('libcap-dev', 'libcap-devel'),
     ('libacl1-dev', 'libacl-devel'),
     ('libattr1-dev', 'libattr-devel'),
+    ('libutf8proc-dev', 'utf8proc-devel'),
+    ('libssl-dev', 'openssl-devel'),
+    ('libclang-dev', 'clang-devel'),
 
     # libNAME1-dev, NAME2-devel
     ('libpopt-dev', 'popt-devel'),
@@ -96,6 +103,7 @@ PKGS = [
     ('libgpgme11-dev', 'gpgme-devel'),
     # NOTE: Debian 8+ and Ubuntu 14.04+
     ('libgnutls28-dev', 'gnutls-devel'),
+    ('gnutls-bin', 'gnutls-utils'),
     ('libtasn1-bin', 'libtasn1-tools'),
     ('libtasn1-dev', 'libtasn1-devel'),
     ('', 'quota-devel'),
@@ -125,12 +133,15 @@ PKGS = [
     ('apt-utils', 'yum-utils'),
     ('pkg-config', 'pkgconfig'),
     ('procps', 'procps-ng'),  # required for the free cmd in tests
-    ('lsb-release', 'lsb-release'),  # we need lsb_relase to show info
+    ('lsb-release', 'lsb-release'),  # we need lsb_release to show info
     ('', 'rpcgen'),  # required for test
     # refer: https://fedoraproject.org/wiki/Changes/SunRPCRemoval
     ('', 'libtirpc-devel'),  # for <rpc/rpc.h> header on fedora
     ('', 'rpcsvc-proto-devel'), # for <rpcsvc/rquota.h> header
     ('mawk', 'gawk'),
+    ('', 'mold'),
+    ('shellcheck', 'ShellCheck'),
+    ('', 'crypto-policies-scripts'),
 
     ('python3', 'python3'),
     ('python3-cryptography', 'python3-cryptography'), # for krb5 tests
@@ -139,19 +150,17 @@ PKGS = [
     ('python3-iso8601', 'python3-iso8601'),
     ('python3-gpg', 'python3-gpg'),  # defaults to ubuntu/fedora latest
     ('python3-markdown', 'python3-markdown'),
-    ('python3-matplotlib', ''),
     ('python3-dnspython', 'python3-dns'),
     ('python3-pexpect', ''),  # for wintest only
     ('python3-pyasn1', 'python3-pyasn1'), # for krb5 tests
     ('python3-setproctitle', 'python3-setproctitle'),
+    ('python3-requests', 'python3-requests'), # for cert auto enroll
 
-    ('', 'libsemanage-python'),
-    ('', 'policycoreutils-python'),
+    ('', 'python3-libsemanage'),
+    ('', 'python3-policycoreutils'),
 
     # perl
     ('libparse-yapp-perl', 'perl-Parse-Yapp'),
-    ('libjson-perl', 'perl-JSON'),
-    ('', 'perl-JSON-Parse'),
     ('perl-modules', ''),
     ('', 'perl-FindBin'),
     ('', 'perl-Archive-Tar'),
@@ -176,8 +185,9 @@ PKGS = [
     # rpm has no pkg for docbook-xml
     ('docbook-xml', 'docbook-dtds'),
     ('docbook-xsl', 'docbook-style-xsl'),
-    ('', 'keyutils-libs-devel'),
+    ('libkeyutils-dev', 'keyutils-libs-devel'),
     ('', 'which'),
+    ('xz-utils', 'xz')
 ]
 
 
@@ -230,7 +240,7 @@ if [ ! -f /usr/bin/python3 ]; then
 fi
 """
 
-CENTOS8S_YUM_BOOTSTRAP = r"""
+ROCKY8_DNF_BOOTSTRAP = r"""
 #!/bin/bash
 {GENERATED_MARKER}
 set -xueo pipefail
@@ -238,20 +248,47 @@ set -xueo pipefail
 yum update -y
 yum install -y dnf-plugins-core
 yum install -y epel-release
+yum install -y centos-release-ceph-pacific
+
+sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-Ceph-*
+sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-Ceph-*
+sed -i 's/$contentdir/centos/g' /etc/yum.repos.d/CentOS-Ceph-*
 
 yum -v repolist all
-yum config-manager --set-enabled powertools -y || \
-    yum config-manager --set-enabled powertools -y
+yum config-manager --set-enabled powertools -y
 
 yum update -y
 
 yum install -y \
     --setopt=install_weak_deps=False \
+    --setopt=centos-ceph-pacific.module_hotfixes=true \
     {pkgs}
 
 yum clean all
 """
 
+CENTOS9S_DNF_BOOTSTRAP = r"""
+#!/bin/bash
+{GENERATED_MARKER}
+set -xueo pipefail
+
+dnf update -y
+dnf install -y dnf-plugins-core
+dnf install -y epel-release
+dnf install -y centos-release-gluster9
+
+dnf -v repolist all
+dnf config-manager --set-enabled crb -y
+
+dnf update -y
+
+dnf install -y \
+    --setopt=install_weak_deps=False \
+    {pkgs}
+
+dnf clean all
+"""
+
 DNF_BOOTSTRAP = r"""
 #!/bin/bash
 {GENERATED_MARKER}
@@ -259,6 +296,25 @@ set -xueo pipefail
 
 dnf update -y
 
+dnf install -y \
+    --setopt=install_weak_deps=False \
+    {pkgs}
+
+dnf clean all
+
+update-crypto-policies --set DEFAULT:AD-SUPPORT
+"""
+
+DNF_BOOTSTRAP_MIT = r"""
+#!/bin/bash
+{GENERATED_MARKER}
+set -xueo pipefail
+
+dnf update -y
+dnf install -y dnf-plugins-core
+dnf copr -y enable abbra/krb5-test
+dnf update -y
+
 dnf install -y \
     --setopt=install_weak_deps=False \
     {pkgs}
@@ -353,7 +409,9 @@ ADD *.sh /tmp/
 RUN /tmp/bootstrap.sh && /tmp/locale.sh
 
 # if ld.gold exists, force link it to ld
-RUN set -x; LD=$(which ld); LD_GOLD=$(which ld.gold); test -x $LD_GOLD && ln -sf $LD_GOLD $LD && test -x $LD && echo "$LD is now $LD_GOLD"
+RUN set -x; ! LD_GOLD=$(which ld.gold) || {{ LD=$(which ld) && ln -sf $LD_GOLD $LD && test -x $LD && echo "$LD is now $LD_GOLD"; }}
+# if ld.mold exists, force link it to ld (prefer mold over gold! ;-)
+RUN set -x; ! LD_MOLD=$(which ld.mold) || {{ LD=$(which ld) && ln -sf $LD_MOLD $LD && test -x $LD && echo "$LD is now $LD_MOLD"; }}
 
 # make test can not work with root, so we have to create a new user
 RUN useradd -m -U -s /bin/bash samba && \
@@ -363,7 +421,7 @@ RUN useradd -m -U -s /bin/bash samba && \
 USER samba
 WORKDIR /home/samba
 # samba tests rely on this
-ENV USER=samba LC_ALL=en_US.utf8 LANG=en_US.utf8
+ENV USER=samba LC_ALL=en_US.utf8 LANG=en_US.utf8 LANGUAGE=en_US
 """
 
 # Vagrantfile snippet for each dist
@@ -388,32 +446,41 @@ Vagrant.configure("2") do |config|
 end
 """
 
-
 DEB_DISTS = {
-    'debian10': {
-        'docker_image': 'debian:10',
-        'vagrant_box': 'debian/buster64',
-        'replace': {
-            'language-pack-en': '',   # included in locales
-            'liburing-dev': '',   # not available
-        }
-    },
     'debian11': {
         'docker_image': 'debian:11',
         'vagrant_box': 'debian/bullseye64',
         'replace': {
             'language-pack-en': '',   # included in locales
+            'shfmt': '',
+            'cargo': '', # included cargo is broken
         }
     },
-    'ubuntu1604': {
-        'docker_image': 'ubuntu:16.04',
-        'vagrant_box': 'ubuntu/xenial64',
+    'debian11-32bit': {
+        'docker_image': 'debian:11',  # specify the platform in .gitlab-ci.yaml
+        'vagrant_box': 'debian/bullseye32',
         'replace': {
-            'python3-gpg': 'python3-gpgme',
-            'glusterfs-common': '',
-            'libcephfs-dev': '',
-            'liburing-dev': '',   # not available
-            'libtracker-sparql-2.0-dev': '', # not available
+            'language-pack-en': '',   # included in locales
+            'shfmt': '',
+            'cargo': '', # included cargo is broken
+        }
+    },
+    'debian12': {
+        'docker_image': 'debian:12',
+        'vagrant_box': 'debian/bookworm64',
+        'replace': {
+            'language-pack-en': '',   # included in locales
+            'libtracker-sparql-2.0-dev': '',  # only tracker 3.x is available
+            'cargo': '', # included cargo is broken
+        }
+    },
+    'debian12-32bit': {
+        'docker_image': 'registry-1.docker.io/i386/debian:12',
+        'vagrant_box': 'debian/bookworm32',
+        'replace': {
+            'language-pack-en': '',   # included in locales
+            'libtracker-sparql-2.0-dev': '',  # only tracker 3.x is available
+            'cargo': '', # included cargo is broken
         }
     },
     'ubuntu1804': {
@@ -421,6 +488,15 @@ DEB_DISTS = {
         'vagrant_box': 'ubuntu/bionic64',
         'replace': {
             'liburing-dev': '',   # not available
+            'shfmt': '',
+        }
+    },
+    'ubuntu1804-32bit': {
+        'docker_image': 'registry-1.docker.io/i386/ubuntu:18.04',
+        'vagrant_box': 'ubuntu/bionic32',
+        'replace': {
+            'liburing-dev': '',   # not available
+            'shfmt': '',
         }
     },
     'ubuntu2004': {
@@ -428,91 +504,71 @@ DEB_DISTS = {
         'vagrant_box': 'ubuntu/focal64',
         'replace': {
             'liburing-dev': '',   # not available
+            'shfmt': '',
         }
     },
+    'ubuntu2204': {
+        'docker_image': 'ubuntu:22.04',
+        'vagrant_box': 'ubuntu/jammy64',
+        'replace': {
+            'libtracker-sparql-2.0-dev': '',  # only tracker 3.x is available
+        },
+    }
 }
 
 
 RPM_DISTS = {
-    'centos7': {
-        'docker_image': 'centos:7',
-        'vagrant_box': 'centos/7',
-        'bootstrap': YUM_BOOTSTRAP,
-        'replace': {
-            'lsb-release': 'redhat-lsb',
-            'python3': 'python36',
-            'python3-cryptography': 'python36-cryptography',
-            'python3-devel': 'python36-devel',
-            'python3-dns': 'python36-dns',
-            'python3-pyasn1': 'python36-pyasn1',
-            'python3-gpg': 'python36-gpg',
-            'python3-iso8601' : 'python36-iso8601',
-            'python3-markdown': 'python36-markdown',
-            # although python36-devel is available
-            # after epel-release installed
-            # however, all other python3 pkgs are still python36-ish
-            'python2-gpg': 'pygpgme',
-            'python3-gpg': '',  # no python3-gpg yet
-            '@development-tools': '"@Development Tools"',  # add quotes
-            'glibc-langpack-en': '',  # included in glibc-common
-            'glibc-locale-source': '',  # included in glibc-common
-            # update perl core modules on centos
-            # fix: Can't locate Archive/Tar.pm in @INC
-            'perl': 'perl-core',
-            'perl-FindBin': '',
-            'rpcsvc-proto-devel': '',
-            'glusterfs-api-devel': '',
-            'glusterfs-devel': '',
-            'libcephfs-devel': '',
-            'gnutls-devel': 'compat-gnutls34-devel',
-            'liburing-devel': '',   # not available
-            'python3-setproctitle': 'python36-setproctitle',
-            'tracker-devel': '', # do not install
-        }
-    },
-    'centos8s': {
-        'docker_image': 'quay.io/centos/centos:stream8',
-        'vagrant_box': 'centos/stream8',
-        'bootstrap': CENTOS8S_YUM_BOOTSTRAP,
+    'rocky8': {
+        'docker_image': 'docker.io/library/rockylinux:8',
+        'vagrant_box': 'rocky/8',
+        'bootstrap': ROCKY8_DNF_BOOTSTRAP,
         'replace': {
             'lsb-release': 'redhat-lsb',
             '@development-tools': '"@Development Tools"',  # add quotes
-            'libsemanage-python': 'python3-libsemanage',
             'lcov': '', # does not exist
             'perl-JSON-Parse': '', # does not exist?
             'perl-Test-Base': 'perl-Test-Simple',
             'perl-FindBin': '',
-            'policycoreutils-python': 'python3-policycoreutils',
             'liburing-devel': '', # not available yet, Add me back, once available!
+            'mold': '',
+            'ShellCheck': '',
+            'shfmt': '',
+            'codespell': '',
         }
     },
-    'fedora33': {
-        'docker_image': 'fedora:33',
-        'vagrant_box': 'fedora/33-cloud-base',
+    'centos9s': {
+        'docker_image': 'quay.io/centos/centos:stream9',
+        'vagrant_box': 'centos/stream9',
+        'bootstrap': CENTOS9S_DNF_BOOTSTRAP,
+        'replace': {
+            'lsb-release': 'lsb_release',
+            '@development-tools': '"@Development Tools"',  # add quotes
+            'lcov': '', # does not exist
+            'perl-JSON-Parse': '', # does not exist?
+            'perl-Test-Base': 'perl-Test-Simple',
+            'perl-FindBin': '',
+            'mold': '',
+            'ShellCheck': '',
+            'shfmt': '',
+            'codespell': '',
+            'libcephfs-devel': '',  # not available anymore
+            'curl': '',  # Use installed curl-minimal
+        }
+    },
+    'fedora41': {
+        'docker_image': 'quay.io/fedora/fedora:41',
+        'vagrant_box': 'fedora/41-cloud-base',
         'bootstrap': DNF_BOOTSTRAP,
         'replace': {
             'lsb-release': 'redhat-lsb',
-            'libsemanage-python': 'python3-libsemanage',
-            'policycoreutils-python': 'python3-policycoreutils',
-            'python3-iso8601': 'python3-dateutil',
-        }
-    },
-    'fedora34': {
-        'docker_image': 'fedora:34',
-        'vagrant_box': 'fedora/34-cloud-base',
-        'bootstrap': DNF_BOOTSTRAP,
-        'replace': {
-            'lsb-release': 'redhat-lsb',
-            'libsemanage-python': 'python3-libsemanage',
-            'policycoreutils-python': 'python3-policycoreutils',
             'perl-FindBin': '',
             'python3-iso8601': 'python3-dateutil',
-            'libtracker-sparql-2.0-dev': '', # only tracker 3.x is available
+            'libtracker-sparql-2.0-dev': '',  # only tracker 3.x is available
         }
     },
-    'opensuse151': {
-        'docker_image': 'opensuse/leap:15.1',
-        'vagrant_box': 'opensuse/openSUSE-15.1-x86_64',
+    'opensuse155': {
+        'docker_image': 'opensuse/leap:15.5',
+        'vagrant_box': 'opensuse/openSUSE-15.5-x86_64',
         'bootstrap': ZYPPER_BOOTSTRAP,
         'replace': {
             '@development-tools': '',
@@ -524,38 +580,7 @@ RPM_DISTS = {
             'jansson-devel': 'libjansson-devel',
             'keyutils-libs-devel': 'keyutils-devel',
             'krb5-workstation': 'krb5-client',
-            'libsemanage-python': 'python2-semanage',
-            'openldap-devel': 'openldap2-devel',
-            'perl-Archive-Tar': 'perl-Archive-Tar-Wrapper',
-            'perl-JSON-Parse': 'perl-JSON-XS',
-            'perl-generators': '',
-            'perl-interpreter': '',
-            'perl-FindBin': '',
-            'procps-ng': 'procps',
-            'python3-dns': 'python3-dnspython',
-            'python3-markdown': 'python3-Markdown',
-            'quota-devel': '',
-            'glusterfs-api-devel': '',
-            'libtasn1-tools': '', # asn1Parser is part of libtasn1
-            'mingw64-gcc': '', # doesn't exist
-            'liburing-devel': '',   # not available
-        }
-    },
-    'opensuse152': {
-        'docker_image': 'opensuse/leap:15.2',
-        'vagrant_box': 'opensuse/openSUSE-15.2-x86_64',
-        'bootstrap': ZYPPER_BOOTSTRAP,
-        'replace': {
-            '@development-tools': '',
-            'dbus-devel': 'dbus-1-devel',
-            'docbook-style-xsl': 'docbook-xsl-stylesheets',
-            'glibc-common': 'glibc-locale',
-            'glibc-locale-source': 'glibc-i18ndata',
-            'glibc-langpack-en': '',
-            'jansson-devel': 'libjansson-devel',
-            'keyutils-libs-devel': 'keyutils-devel',
-            'krb5-workstation': 'krb5-client',
-            'libsemanage-python': 'python2-semanage',
+            'python3-libsemanage': 'python3-semanage',
             'openldap-devel': 'openldap2-devel',
             'perl-Archive-Tar': 'perl-Archive-Tar-Wrapper',
             'perl-JSON-Parse': 'perl-JSON-XS',
@@ -568,7 +593,11 @@ RPM_DISTS = {
             'python3-markdown': 'python3-Markdown',
             'quota-devel': '',
             'glusterfs-api-devel': '',
+            'gnutls-utils': 'gnutls',
             'libtasn1-tools': '', # asn1Parser is part of libtasn1
+            'mold': '',
+            'shfmt': '',
+            'yum-utils': '',
         }
     }
 }
