@@ -132,7 +132,7 @@
  *
  * The ad_xattr_header follows the FinderInfo data in the FinderInfo entry if
  * the length of the FinderInfo entry is larger then 32 bytes. It is then
- * preceeded with 2 bytes padding.
+ * preceded with 2 bytes padding.
  *
  * Cf: https://opensource.apple.com/source/xnu/xnu-4570.1.46/bsd/vfs/vfs_xattr.c
  */
@@ -210,7 +210,7 @@ static const uint32_t set_eid[] = {
 	AD_DEV, AD_INO, AD_SYN, AD_ID
 };
 
-static char empty_resourcefork[] = {
+static const char empty_resourcefork[] = {
 	0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00,
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1E,
 	0x54, 0x68, 0x69, 0x73, 0x20, 0x72, 0x65, 0x73,
@@ -282,7 +282,7 @@ static bool ad_entry_check_size(uint32_t eid,
 				uint32_t off,
 				uint32_t got_len)
 {
-	struct {
+	static const struct {
 		off_t expected_len;
 		bool fixed_size;
 		bool minimum_size;
@@ -328,7 +328,7 @@ static bool ad_entry_check_size(uint32_t eid,
 	}
 	if (ad_checks[eid].fixed_size) {
 		if (ad_checks[eid].expected_len != got_len) {
-			/* Wrong size fo fixed size entry. */
+			/* Wrong size for fixed size entry. */
 			return false;
 		}
 	} else {
@@ -493,7 +493,7 @@ static bool ad_pack_move_reso(struct vfs_handle_struct *handle,
 		 * This buffer is already set when converting a resourcefork
 		 * stream from vfs_streams_depot backend via ad_unconvert(). It
 		 * is NULL with vfs_streams_xattr where the resourcefork stream
-		 * is stored in an AppleDouble sidecar file vy vfs_fruit.
+		 * is stored in an AppleDouble sidecar file by vfs_fruit.
 		 */
 		ad->ad_rsrc_data = talloc_size(ad, reso_len);
 		if (ad->ad_rsrc_data == NULL) {
@@ -949,7 +949,7 @@ static bool ad_unpack_xattrs(struct adouble *ad)
 }
 
 /**
- * Unpack an AppleDouble blob into a struct adoble
+ * Unpack an AppleDouble blob into a struct adouble
  **/
 static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		      size_t filesize)
@@ -967,14 +967,14 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 	 */
 
 	if (bufsize < (AD_HEADER_LEN + (AD_ENTRY_LEN * nentries))) {
-		DEBUG(1, ("bad size\n"));
+		DBG_NOTICE("Bad size\n");
 		return false;
 	}
 
 	ad->ad_magic = RIVAL(ad->ad_data, 0);
 	ad->ad_version = RIVAL(ad->ad_data, ADEDOFF_VERSION);
 	if ((ad->ad_magic != AD_MAGIC) || (ad->ad_version != AD_VERSION)) {
-		DEBUG(1, ("wrong magic or version\n"));
+		DBG_NOTICE("Wrong magic or version\n");
 		return false;
 	}
 
@@ -982,8 +982,7 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 
 	adentries = RSVAL(ad->ad_data, ADEDOFF_NENTRIES);
 	if (adentries != nentries) {
-		DEBUG(1, ("invalid number of entries: %zu\n",
-			  adentries));
+		DBG_NOTICE("Invalid number of entries: %zu\n", adentries);
 		return false;
 	}
 
@@ -995,7 +994,7 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		len = RIVAL(ad->ad_data, AD_HEADER_LEN + (i * AD_ENTRY_LEN) + 8);
 
 		if (!eid || eid >= ADEID_MAX) {
-			DEBUG(1, ("bogus eid %d\n", eid));
+			DBG_NOTICE("Bogus eid %d\n", eid);
 			return false;
 		}
 
@@ -1005,16 +1004,22 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		 * ensure the specified offset is within that bound
 		 */
 		if ((off > bufsize) && (eid != ADEID_RFORK)) {
-			DEBUG(1, ("bogus eid %d: off: %" PRIu32 ", len: %" PRIu32 "\n",
-				  eid, off, len));
+			DBG_NOTICE("Bogus eid %d: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   eid,
+				   off,
+				   len);
 			return false;
 		}
 
 		ok = ad_entry_check_size(eid, bufsize, off, len);
 		if (!ok) {
-			DBG_ERR("bogus eid [%"PRIu32"] bufsize [%zu] "
-				"off [%"PRIu32"] len [%"PRIu32"]\n",
-				eid, bufsize, off, len);
+			DBG_NOTICE("Bogus eid [%" PRIu32 "] bufsize [%zu] "
+				   "off [%" PRIu32 "] len [%" PRIu32 "]\n",
+				   eid,
+				   bufsize,
+				   off,
+				   len);
 			return false;
 		}
 
@@ -1022,8 +1027,11 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		 * That would be obviously broken
 		 */
 		if (off > filesize) {
-			DEBUG(1, ("bogus eid %d: off: %" PRIu32 ", len: %" PRIu32 "\n",
-				  eid, off, len));
+			DBG_NOTICE("Bogus eid %d: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   eid,
+				   off,
+				   len);
 			return false;
 		}
 
@@ -1032,9 +1040,11 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 		 * filesize.
 		 */
 		if (off + len < off) {
-			DEBUG(1, ("offset wrap in eid %d: off: %" PRIu32
-				  ", len: %" PRIu32 "\n",
-				  eid, off, len));
+			DBG_NOTICE("offset wrap in eid %d: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   eid,
+				   off,
+				   len);
 			return false;
 
 		}
@@ -1045,9 +1055,11 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 			 * out.
 			 */
 			if (eid != ADEID_RFORK) {
-				DEBUG(1, ("bogus eid %d: off: %" PRIu32
-					  ", len: %" PRIu32 "\n",
-					  eid, off, len));
+				DBG_NOTICE("Bogus eid %d: off: %" PRIu32
+					   ", len: %" PRIu32 "\n",
+					   eid,
+					   off,
+					   len);
 				return false;
 			}
 
@@ -1056,8 +1068,10 @@ static bool ad_unpack(struct adouble *ad, const size_t nentries,
 			 * the size to entryoffset - filesize.
 			 */
 			len = filesize - off;
-			DEBUG(1, ("Limiting ADEID_RFORK: off: %" PRIu32
-				  ", len: %" PRIu32 "\n", off, len));
+			DBG_NOTICE("Limiting ADEID_RFORK: off: %" PRIu32
+				   ", len: %" PRIu32 "\n",
+				   off,
+				   len);
 		}
 
 		ad->ad_eid[eid].ade_off = off;
@@ -1171,16 +1185,15 @@ static bool ad_convert_xattr(vfs_handle_struct *handle,
 		files_struct *fsp = NULL;
 		ssize_t nwritten;
 
-		status = string_replace_allocate(handle->conn,
-						 e->adx_name,
-						 string_replace_cmaps,
-						 talloc_tos(),
-						 &mapped_name,
-						 vfs_translate_to_windows);
-		if (!NT_STATUS_IS_OK(status) &&
-		    !NT_STATUS_EQUAL(status, NT_STATUS_NONE_MAPPED))
-		{
-			DBG_ERR("string_replace_allocate failed\n");
+		rc = string_replace_allocate(handle->conn,
+					     e->adx_name,
+					     string_replace_cmaps,
+					     talloc_tos(),
+					     &mapped_name,
+					     vfs_translate_to_windows);
+		if (rc != 0) {
+			DBG_ERR("string_replace_allocate failed: %s\n",
+				strerror(rc));
 			ok = false;
 			goto fail;
 		}
@@ -1208,12 +1221,6 @@ static bool ad_convert_xattr(vfs_handle_struct *handle,
 
 		DBG_DEBUG("stream_name: %s\n", smb_fname_str_dbg(stream_name));
 
-		rc = vfs_stat(handle->conn, stream_name);
-		if (rc == -1 && errno != ENOENT) {
-			ok = false;
-			goto fail;
-		}
-
 		status = openat_pathref_fsp(handle->conn->cwd_fsp, stream_name);
 		if (!NT_STATUS_IS_OK(status) &&
 		    !NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND))
@@ -1225,9 +1232,10 @@ static bool ad_convert_xattr(vfs_handle_struct *handle,
 		status = SMB_VFS_CREATE_FILE(
 			handle->conn,			/* conn */
 			NULL,				/* req */
+			NULL,				/* dirfsp */
 			stream_name,			/* fname */
 			FILE_GENERIC_WRITE,		/* access_mask */
-			FILE_SHARE_READ | FILE_SHARE_WRITE, /* share_access */
+			FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, /* share_access */
 			FILE_OPEN_IF,			/* create_disposition */
 			0,				/* create_options */
 			0,				/* file_attributes */
@@ -1254,13 +1262,13 @@ static bool ad_convert_xattr(vfs_handle_struct *handle,
 		if (nwritten == -1) {
 			DBG_ERR("SMB_VFS_PWRITE failed\n");
 			saved_errno = errno;
-			close_file(NULL, fsp, ERROR_CLOSE);
+			close_file_free(NULL, &fsp, ERROR_CLOSE);
 			errno = saved_errno;
 			ok = false;
 			goto fail;
 		}
 
-		status = close_file(NULL, fsp, NORMAL_CLOSE);
+		status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		if (!NT_STATUS_IS_OK(status)) {
 			ok = false;
 			goto fail;
@@ -1307,7 +1315,6 @@ static bool ad_convert_finderinfo(vfs_handle_struct *handle,
 	NTSTATUS status;
 	int saved_errno = 0;
 	int cmp;
-	int rc;
 
 	cmp = memcmp(ad->ad_filler, AD_FILLER_TAG_OSX, ADEDLEN_FILLER);
 	if (cmp != 0) {
@@ -1352,11 +1359,6 @@ static bool ad_convert_finderinfo(vfs_handle_struct *handle,
 
 	DBG_DEBUG("stream_name: %s\n", smb_fname_str_dbg(stream_name));
 
-	rc = vfs_stat(handle->conn, stream_name);
-	if (rc == -1 && errno != ENOENT) {
-		return false;
-	}
-
 	status = openat_pathref_fsp(handle->conn->cwd_fsp, stream_name);
 	if (!NT_STATUS_IS_OK(status) &&
 	    !NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND))
@@ -1367,6 +1369,7 @@ static bool ad_convert_finderinfo(vfs_handle_struct *handle,
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,			/* conn */
 		NULL,				/* req */
+		NULL,				/* dirfsp */
 		stream_name,			/* fname */
 		FILE_GENERIC_WRITE,		/* access_mask */
 		FILE_SHARE_READ | FILE_SHARE_WRITE, /* share_access */
@@ -1395,12 +1398,12 @@ static bool ad_convert_finderinfo(vfs_handle_struct *handle,
 	if (nwritten == -1) {
 		DBG_ERR("SMB_VFS_PWRITE failed\n");
 		saved_errno = errno;
-		close_file(NULL, fsp, ERROR_CLOSE);
+		close_file_free(NULL, &fsp, ERROR_CLOSE);
 		errno = saved_errno;
 		return false;
 	}
 
-	status = close_file(NULL, fsp, NORMAL_CLOSE);
+	status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 	if (!NT_STATUS_IS_OK(status)) {
 		return false;
 	}
@@ -1607,21 +1610,16 @@ static bool ad_unconvert_open_ad(TALLOC_CTX *mem_ctx,
 	NTSTATUS status;
 	int ret;
 
-	ret = vfs_stat(handle->conn, adpath);
-	if (ret == -1 && errno != ENOENT) {
+	status = openat_pathref_fsp(handle->conn->cwd_fsp, adpath);
+	if (!NT_STATUS_IS_OK(status) &&
+	    !NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND)) {
 		return false;
 	}
 
-	if (VALID_STAT(adpath->st)) {
-		status = openat_pathref_fsp(handle->conn->cwd_fsp, adpath);
-		if (!NT_STATUS_IS_OK(status)) {
-			return false;
-		}
-	}
-
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,
 		NULL,				/* req */
+		NULL,				/* dirfsp */
 		adpath,
 		FILE_READ_DATA|FILE_WRITE_DATA,
 		FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
@@ -1652,7 +1650,7 @@ static bool ad_unconvert_open_ad(TALLOC_CTX *mem_ctx,
 		if (ret != 0) {
 			DBG_ERR("SMB_VFS_FCHOWN [%s] failed: %s\n",
 				fsp_str_dbg(fsp), nt_errstr(status));
-			close_file(NULL, fsp, NORMAL_CLOSE);
+			close_file_free(NULL, &fsp, NORMAL_CLOSE);
 			return false;
 		}
 	}
@@ -1670,10 +1668,6 @@ static bool ad_unconvert_get_streams(struct vfs_handle_struct *handle,
 	files_struct *fsp = NULL;
 	NTSTATUS status;
 
-	if (!VALID_STAT(smb_fname->st)) {
-		return false;
-	}
-
 	status = openat_pathref_fsp(handle->conn->cwd_fsp, smb_fname);
 	if (!NT_STATUS_IS_OK(status)) {
 		return false;
@@ -1682,6 +1676,7 @@ static bool ad_unconvert_get_streams(struct vfs_handle_struct *handle,
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,				/* conn */
 		NULL,					/* req */
+		NULL,					/* dirfsp */
 		smb_fname,				/* fname */
 		FILE_READ_ATTRIBUTES,			/* access_mask */
 		(FILE_SHARE_READ | FILE_SHARE_WRITE |	/* share_access */
@@ -1710,14 +1705,14 @@ static bool ad_unconvert_get_streams(struct vfs_handle_struct *handle,
 				num_streams,
 				streams);
 	if (!NT_STATUS_IS_OK(status)) {
-		close_file(NULL, fsp, NORMAL_CLOSE);
+		close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		DBG_ERR("streaminfo on [%s] failed: %s\n",
 			smb_fname_str_dbg(smb_fname),
 			nt_errstr(status));
 		return false;
 	}
 
-	status = close_file(NULL, fsp, NORMAL_CLOSE);
+	status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("close_file [%s] failed: %s\n",
 			smb_fname_str_dbg(smb_fname),
@@ -1749,8 +1744,8 @@ static bool ad_collect_one_stream(struct vfs_handle_struct *handle,
 	size_t needed_size;
 	ssize_t nread;
 	NTSTATUS status;
-	int ret;
 	bool ok;
+	int rc;
 
 	sname = synthetic_smb_fname(ad,
 				    smb_fname->base_name,
@@ -1769,13 +1764,6 @@ static bool ad_collect_one_stream(struct vfs_handle_struct *handle,
 
 	DBG_DEBUG("Collecting stream [%s]\n", smb_fname_str_dbg(sname));
 
-	ret = SMB_VFS_STAT(handle->conn, sname);
-	if (ret != 0) {
-		DBG_ERR("SMB_VFS_STAT [%s] failed\n", smb_fname_str_dbg(sname));
-		ok = false;
-		goto out;
-	}
-
 	status = openat_pathref_fsp(handle->conn->cwd_fsp, sname);
 	if (!NT_STATUS_IS_OK(status)) {
 		ok = false;
@@ -1785,6 +1773,7 @@ static bool ad_collect_one_stream(struct vfs_handle_struct *handle,
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,
 		NULL,				/* req */
+		NULL,				/* dirfsp */
 		sname,
 		FILE_READ_DATA|DELETE_ACCESS,
 		FILE_SHARE_READ,
@@ -1912,16 +1901,14 @@ static bool ad_collect_one_stream(struct vfs_handle_struct *handle,
 		*p = '\0';
 	}
 
-	status = string_replace_allocate(handle->conn,
-					 e->adx_name,
-					 cmaps,
-					 ad,
-					 &mapped_name,
-					 vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status) &&
-	    !NT_STATUS_EQUAL(status, NT_STATUS_NONE_MAPPED))
-	{
-		DBG_ERR("string_replace_allocate failed\n");
+	rc = string_replace_allocate(handle->conn,
+				     e->adx_name,
+				     cmaps,
+				     ad,
+				     &mapped_name,
+				     vfs_translate_to_unix);
+	if (rc != 0) {
+		DBG_ERR("string_replace_allocate failed: %s\n", strerror(rc));
 		ok = false;
 		goto out;
 	}
@@ -1975,7 +1962,7 @@ static bool ad_collect_one_stream(struct vfs_handle_struct *handle,
 out:
 	TALLOC_FREE(sname);
 	if (fsp != NULL) {
-		status = close_file(NULL, fsp, NORMAL_CLOSE);
+		status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		if (!NT_STATUS_IS_OK(status)) {
 			DBG_ERR("close_file [%s] failed: %s\n",
 				smb_fname_str_dbg(smb_fname),
@@ -2117,9 +2104,9 @@ bool ad_unconvert(TALLOC_CTX *mem_ctx,
 
 out:
 	if (fsp != NULL) {
-		status = close_file(NULL, fsp, NORMAL_CLOSE);
+		status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		if (!NT_STATUS_IS_OK(status)) {
-			DBG_ERR("close_file [%s] failed: %s\n",
+			DBG_ERR("close_file_free() [%s] failed: %s\n",
 				smb_fname_str_dbg(smb_fname),
 				nt_errstr(status));
 			ok = false;
@@ -2143,9 +2130,7 @@ static ssize_t ad_read_meta(vfs_handle_struct *handle,
 
 	DEBUG(10, ("reading meta xattr for %s\n", smb_fname->base_name));
 
-	if (fsp->base_fsp != NULL) {
-		fsp = fsp->base_fsp;
-	}
+	fsp = metadata_fsp(fsp);
 
 	ealen = SMB_VFS_FGETXATTR(fsp,
 				  AFPINFO_EA_NETATALK,
@@ -2178,7 +2163,11 @@ static ssize_t ad_read_meta(vfs_handle_struct *handle,
 	/* Now parse entries */
 	ok = ad_unpack(ad, ADEID_NUM_XATTR, AD_DATASZ_XATTR);
 	if (!ok) {
-		DEBUG(2, ("invalid AppleDouble metadata xattr\n"));
+		DBG_WARNING(
+			"Invalid AppleDouble xattr metadata (%s) in file: %s - "
+			"Consider deleting the corrupted file.\n",
+			smb_fname->base_name,
+			ad->ad_fsp->fsp_name->base_name);
 		errno = EINVAL;
 		rc = -1;
 		goto exit;
@@ -2226,7 +2215,7 @@ static NTSTATUS adouble_open_rsrc_fsp(const struct files_struct *dirfsp,
 	struct smb_filename *adp_smb_fname = NULL;
 	struct files_struct *ad_fsp = NULL;
 	NTSTATUS status;
-	int flags = in_flags;
+	struct vfs_open_how how = { .flags = in_flags, .mode = mode, };
 
 	rc = adouble_path(talloc_tos(),
 			  smb_base_fname,
@@ -2243,25 +2232,24 @@ static NTSTATUS adouble_open_rsrc_fsp(const struct files_struct *dirfsp,
 	}
 
 #ifdef O_PATH
-	flags &= ~(O_PATH);
+	how.flags &= ~(O_PATH);
 #endif
-	if (flags & (O_CREAT | O_TRUNC | O_WRONLY)) {
+	if (how.flags & (O_CREAT | O_TRUNC | O_WRONLY)) {
 		/* We always need read/write access for the metadata header too */
-		flags &= ~(O_WRONLY);
-		flags |= O_RDWR;
+		how.flags &= ~(O_WRONLY);
+		how.flags |= O_RDWR;
 	}
 
 	status = fd_openat(dirfsp,
 			   adp_smb_fname,
 			   ad_fsp,
-			   flags,
-			   mode);
+			   &how);
 	if (!NT_STATUS_IS_OK(status)) {
 		file_free(NULL, ad_fsp);
 		return status;
 	}
 
-	if (flags & (O_CREAT | O_TRUNC)) {
+	if (how.flags & (O_CREAT | O_TRUNC)) {
 		ad = ad_init(talloc_tos(), ADOUBLE_RSRC);
 		if (ad == NULL) {
 			file_free(NULL, ad_fsp);
@@ -2290,7 +2278,7 @@ NTSTATUS adouble_open_from_base_fsp(const struct files_struct *dirfsp,
 	*_ad_fsp = NULL;
 
 	SMB_ASSERT(base_fsp != NULL);
-	SMB_ASSERT(base_fsp->base_fsp == NULL);
+	SMB_ASSERT(!fsp_is_alternate_stream(base_fsp));
 
 	switch (type) {
 	case ADOUBLE_META:
@@ -2388,8 +2376,10 @@ static ssize_t ad_read_rsrc_adouble(vfs_handle_struct *handle,
 		       ADEID_NUM_DOT_UND,
 		       ad->ad_fsp->fsp_name->st.st_ex_size);
 	if (!ok) {
-		DBG_ERR("invalid AppleDouble resource %s\n",
-			smb_fname->base_name);
+		DBG_WARNING("Invalid AppleDouble resource (%s) in file: %s - "
+			    "Consider deleting the corrupted file.\n",
+			    smb_fname->base_name,
+			    ad->ad_fsp->fsp_name->base_name);
 		errno = EINVAL;
 		return -1;
 	}
@@ -2398,8 +2388,10 @@ static ssize_t ad_read_rsrc_adouble(vfs_handle_struct *handle,
 	    || (ad_getentrylen(ad, ADEID_FINDERI) < ADEDLEN_FINDERI)
 	    || (ad_getentryoff(ad, ADEID_RFORK) < ADEDOFF_RFORK_DOT_UND))
 	{
-		DBG_ERR("invalid AppleDouble resource %s\n",
-			smb_fname->base_name);
+		DBG_WARNING("Invalid AppleDouble resource (%s) in file: %s - "
+			    "Consider deleting the corrupted file.\n",
+			    smb_fname->base_name,
+			    ad->ad_fsp->fsp_name->base_name);
 		errno = EINVAL;
 		return -1;
 	}
@@ -2457,7 +2449,7 @@ static int adouble_destructor(struct adouble *ad)
 }
 
 /**
- * Allocate a struct adouble without initialiing it
+ * Allocate a struct adouble without initializing it
  *
  * The struct is either hang of the fsp extension context or if fsp is
  * NULL from ctx.
@@ -2587,14 +2579,10 @@ static struct adouble *ad_get_internal(TALLOC_CTX *ctx,
 	int rc = 0;
 	ssize_t len;
 	struct adouble *ad = NULL;
-	int mode;
 
 	if (fsp != NULL) {
-		if (fsp->base_fsp != NULL) {
-			smb_fname = fsp->base_fsp->fsp_name;
-		} else {
-			smb_fname = fsp->fsp_name;
-		}
+		struct files_struct *meta_fsp = metadata_fsp(fsp);
+		smb_fname = meta_fsp->fsp_name;
 	}
 
 	DEBUG(10, ("ad_get(%s) called for %s\n",
@@ -2608,12 +2596,9 @@ static struct adouble *ad_get_internal(TALLOC_CTX *ctx,
 	}
 
 	/* Try rw first so we can use the fd in ad_convert() */
-	mode = O_RDWR;
-
-	rc = ad_open(handle, ad, fsp, smb_fname, mode, 0);
+	rc = ad_open(handle, ad, fsp, smb_fname, O_RDWR, 0);
 	if (rc == -1 && ((errno == EROFS) || (errno == EACCES))) {
-		mode = O_RDONLY;
-		rc = ad_open(handle, ad, fsp, smb_fname, mode, 0);
+		rc = ad_open(handle, ad, fsp, smb_fname, O_RDONLY, 0);
 	}
 	if (rc == -1) {
 		DBG_DEBUG("ad_open [%s] error [%s]\n",
@@ -2713,7 +2698,7 @@ int ad_fset(struct vfs_handle_struct *handle,
 					  ad_getentryoff(ad, ADEID_RFORK),
 					  0);
 		if (len != ad_getentryoff(ad, ADEID_RFORK)) {
-			DBG_ERR("short write on %s: %zd", fsp_str_dbg(fsp), len);
+			DBG_ERR("short write on %s: %zd\n", fsp_str_dbg(fsp), len);
 			return -1;
 		}
 		rc = 0;
@@ -2759,17 +2744,14 @@ int adouble_path(TALLOC_CTX *ctx,
 {
 	char *parent;
 	const char *base;
-	struct smb_filename *smb_fname = cp_smb_filename(ctx,
-						smb_fname_in);
+	struct smb_filename *smb_fname = NULL;
 
+	smb_fname = cp_smb_filename_nostream(ctx, smb_fname_in);
 	if (smb_fname == NULL) {
 		return -1;
 	}
 
-	/* We need streamname to be NULL */
-	TALLOC_FREE(smb_fname->stream_name);
-
-	/* And we're replacing base_name. */
+	/* We're replacing base_name. */
 	TALLOC_FREE(smb_fname->base_name);
 
 	SET_STAT_INVALID(smb_fname->st);
@@ -2836,7 +2818,7 @@ ssize_t afpinfo_pack(const AfpInfo *ai, char *buf)
  * Buffer size must be at least AFP_INFO_SIZE
  * Returns allocated AfpInfo struct
  **/
-AfpInfo *afpinfo_unpack(TALLOC_CTX *ctx, const void *data)
+AfpInfo *afpinfo_unpack(TALLOC_CTX *ctx, const void *data, bool validate)
 {
 	AfpInfo *ai = talloc_zero(ctx, AfpInfo);
 	if (ai == NULL) {
@@ -2849,10 +2831,16 @@ AfpInfo *afpinfo_unpack(TALLOC_CTX *ctx, const void *data)
 	memcpy(ai->afpi_FinderInfo, (const char *)data + 16,
 	       sizeof(ai->afpi_FinderInfo));
 
-	if (ai->afpi_Signature != AFP_Signature
-	    || ai->afpi_Version != AFP_Version) {
-		DEBUG(1, ("Bad AfpInfo signature or version\n"));
-		TALLOC_FREE(ai);
+	if (validate) {
+		if (ai->afpi_Signature != AFP_Signature
+		    || ai->afpi_Version != AFP_Version)
+		{
+			DEBUG(1, ("Bad AfpInfo signature or version\n"));
+			TALLOC_FREE(ai);
+		}
+	} else {
+		ai->afpi_Signature = AFP_Signature;
+		ai->afpi_Version = AFP_Version;
 	}
 
 	return ai;
