@@ -131,31 +131,31 @@ static bool init_mappings(connection_struct *conn,
 	return False;
 }
 
-static NTSTATUS catia_string_replace_allocate(connection_struct *conn,
-					      const char *name_in,
-					      char **mapped_name,
-					enum vfs_translate_direction direction)
+static int catia_string_replace_allocate(
+	connection_struct *conn,
+	const char *name_in,
+	char **mapped_name,
+	enum vfs_translate_direction direction)
 {
 	struct share_mapping_entry *selected;
-	NTSTATUS status;
+	int ret;
 
 	if (!init_mappings(conn, &selected)) {
 		/* No mappings found. Just use the old name */
 		*mapped_name = talloc_strdup(talloc_tos(), name_in);
 		if (!*mapped_name) {
-			errno = ENOMEM;
-			return NT_STATUS_NO_MEMORY;
+			return ENOMEM;
 		}
-		return NT_STATUS_OK;
+		return 0;
 	}
 
-	status = string_replace_allocate(conn,
-					 name_in,
-					 selected->mappings,
-					 talloc_tos(),
-					 mapped_name,
-					 direction);
-	return status;
+	ret = string_replace_allocate(conn,
+				      name_in,
+				      selected->mappings,
+				      talloc_tos(),
+				      mapped_name,
+				      direction);
+	return ret;
 }
 
 static int catia_connect(struct vfs_handle_struct *handle,
@@ -183,7 +183,8 @@ static NTSTATUS catia_translate_name(struct vfs_handle_struct *handle,
 {
 	char *name = NULL;
 	char *mapped_name;
-	NTSTATUS status, ret;
+	NTSTATUS ret;
+	int rc;
 
 	/*
 	 * Copy the supplied name and free the memory for mapped_name,
@@ -196,12 +197,14 @@ static NTSTATUS catia_translate_name(struct vfs_handle_struct *handle,
 		errno = ENOMEM;
 		return NT_STATUS_NO_MEMORY;
 	}
-	status = catia_string_replace_allocate(handle->conn, name,
-			&mapped_name, direction);
+	rc = catia_string_replace_allocate(handle->conn,
+					   name,
+					   &mapped_name,
+					   direction);
 
 	TALLOC_FREE(name);
-	if (!NT_STATUS_IS_OK(status)) {
-		return status;
+	if (rc != 0) {
+		return map_nt_error_from_unix(rc);
 	}
 
 	ret = SMB_VFS_NEXT_TRANSLATE_NAME(handle, mapped_name, direction,
@@ -210,7 +213,7 @@ static NTSTATUS catia_translate_name(struct vfs_handle_struct *handle,
 	if (NT_STATUS_EQUAL(ret, NT_STATUS_NONE_MAPPED)) {
 		*pmapped_name = talloc_move(mem_ctx, &mapped_name);
 		/* we need to return the former translation result here */
-		ret = status;
+		ret = NT_STATUS_OK;
 	} else {
 		TALLOC_FREE(mapped_name);
 	}
@@ -286,7 +289,7 @@ static struct catia_cache *catia_validate_and_apply_cc(
 
 		if ((cc->fname != fsp->fsp_name->base_name)
 		    ||
-		    ((fsp->base_fsp != NULL) &&
+		    (fsp_is_alternate_stream(fsp) &&
 		     (cc->base_fname != fsp->base_fsp->fsp_name->base_name)))
 		{
 			CATIA_DEBUG_CC(10, cc, fsp);
@@ -312,7 +315,7 @@ static struct catia_cache *catia_validate_and_apply_cc(
 
 	if ((cc->orig_fname != fsp->fsp_name->base_name)
 	    ||
-	    ((fsp->base_fsp != NULL) &&
+	    (fsp_is_alternate_stream(fsp) &&
 	     (cc->orig_base_fname != fsp->base_fsp->fsp_name->base_name)))
 	{
 		/*
@@ -331,7 +334,7 @@ static struct catia_cache *catia_validate_and_apply_cc(
 	 * names from the cache and mark the cc as busy.
 	 */
 	fsp->fsp_name->base_name = cc->fname;
-	if (fsp->base_fsp != NULL) {
+	if (fsp_is_alternate_stream(fsp)) {
 		fsp->base_fsp->fsp_name->base_name = cc->base_fname;
 	}
 
@@ -352,8 +355,8 @@ static int catia_fetch_fsp_pre_next(TALLOC_CTX *mem_ctx,
 	const struct catia_cache * const *busy =
 		(const struct catia_cache * const *)_cc;
 	struct catia_cache *cc = NULL;
-	NTSTATUS status;
 	bool make_tmp_cache = false;
+	int ret;
 
 	*_cc = NULL;
 
@@ -395,27 +398,26 @@ static int catia_fetch_fsp_pre_next(TALLOC_CTX *mem_ctx,
 		mem_ctx = cc;
 	}
 
-
-	status = catia_string_replace_allocate(handle->conn,
-					       fsp->fsp_name->base_name,
-					       &cc->fname,
-					       vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
+	ret = catia_string_replace_allocate(handle->conn,
+					    fsp->fsp_name->base_name,
+					    &cc->fname,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
 		catia_free_cc(&cc, handle, fsp);
-		errno = map_errno_from_nt_status(status);
+		errno = ret;
 		return -1;
 	}
 	talloc_steal(mem_ctx, cc->fname);
 
-	if (fsp->base_fsp != NULL) {
-		status = catia_string_replace_allocate(
+	if (fsp_is_alternate_stream(fsp)) {
+		ret = catia_string_replace_allocate(
 			handle->conn,
 			fsp->base_fsp->fsp_name->base_name,
 			&cc->base_fname,
 			vfs_translate_to_unix);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (ret != 0) {
 			catia_free_cc(&cc, handle, fsp);
-			errno = map_errno_from_nt_status(status);
+			errno = ret;
 			return -1;
 		}
 		talloc_steal(mem_ctx, cc->base_fname);
@@ -424,7 +426,7 @@ static int catia_fetch_fsp_pre_next(TALLOC_CTX *mem_ctx,
 	cc->orig_fname = fsp->fsp_name->base_name;
 	fsp->fsp_name->base_name = cc->fname;
 
-	if (fsp->base_fsp != NULL) {
+	if (fsp_is_alternate_stream(fsp)) {
 		cc->orig_base_fname = fsp->base_fsp->fsp_name->base_name;
 		fsp->base_fsp->fsp_name->base_name = cc->base_fname;
 	}
@@ -472,7 +474,7 @@ static void catia_fetch_fsp_post_next(struct catia_cache **_cc,
 	*_cc = NULL;
 
 	fsp->fsp_name->base_name = cc->orig_fname;
-	if (fsp->base_fsp != NULL) {
+	if (fsp_is_alternate_stream(fsp)) {
 		fsp->base_fsp->fsp_name->base_name = cc->orig_base_fname;
 	}
 
@@ -489,21 +491,20 @@ static int catia_openat(vfs_handle_struct *handle,
 			const struct files_struct *dirfsp,
 			const struct smb_filename *smb_fname_in,
 			files_struct *fsp,
-			int flags,
-			mode_t mode)
+			const struct vfs_open_how *how)
 {
 	struct smb_filename *smb_fname = NULL;
 	struct catia_cache *cc = NULL;
 	char *mapped_name = NULL;
-	NTSTATUS status;
 	int ret;
 	int saved_errno = 0;
 
-	status = catia_string_replace_allocate(handle->conn,
-					       smb_fname_in->base_name,
-					       &mapped_name,
-					       vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname_in->base_name,
+					    &mapped_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
@@ -525,9 +526,7 @@ static int catia_openat(vfs_handle_struct *handle,
 				  dirfsp,
 				  smb_fname,
 				  fsp,
-				  flags,
-				  mode);
-
+				  how);
 	if (ret == -1) {
 		saved_errno = errno;
 	}
@@ -544,42 +543,48 @@ static int catia_renameat(vfs_handle_struct *handle,
 			files_struct *srcfsp,
 			const struct smb_filename *smb_fname_src,
 			files_struct *dstfsp,
-			const struct smb_filename *smb_fname_dst)
+			const struct smb_filename *smb_fname_dst,
+			const struct vfs_rename_how *how)
 {
 	TALLOC_CTX *ctx = talloc_tos();
 	struct smb_filename *smb_fname_src_tmp = NULL;
 	struct smb_filename *smb_fname_dst_tmp = NULL;
 	char *src_name_mapped = NULL;
 	char *dst_name_mapped = NULL;
-	NTSTATUS status;
 	int ret = -1;
 
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname_src->base_name,
-				&src_name_mapped, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname_src->base_name,
+					    &src_name_mapped,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname_dst->base_name,
-				&dst_name_mapped, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
-		return -1;
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname_dst->base_name,
+					    &dst_name_mapped,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		TALLOC_FREE(src_name_mapped);
+		errno = ret;
+		ret = -1;
+		goto out;
 	}
 
 	/* Setup temporary smb_filename structs. */
 	smb_fname_src_tmp = cp_smb_filename(ctx, smb_fname_src);
 	if (smb_fname_src_tmp == NULL) {
 		errno = ENOMEM;
+		ret = -1;
 		goto out;
 	}
 
 	smb_fname_dst_tmp = cp_smb_filename(ctx, smb_fname_dst);
 	if (smb_fname_dst_tmp == NULL) {
 		errno = ENOMEM;
+		ret = -1;
 		goto out;
 	}
 
@@ -594,7 +599,8 @@ static int catia_renameat(vfs_handle_struct *handle,
 			srcfsp,
 			smb_fname_src_tmp,
 			dstfsp,
-			smb_fname_dst_tmp);
+			smb_fname_dst_tmp,
+			how);
 
 out:
 	TALLOC_FREE(src_name_mapped);
@@ -611,13 +617,13 @@ static int catia_stat(vfs_handle_struct *handle,
 	char *name = NULL;
 	char *tmp_base_name;
 	int ret;
-	NTSTATUS status;
 
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname->base_name,
-				&name, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
@@ -637,13 +643,13 @@ static int catia_lstat(vfs_handle_struct *handle,
 	char *name = NULL;
 	char *tmp_base_name;
 	int ret;
-	NTSTATUS status;
 
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname->base_name,
-				&name, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
@@ -665,7 +671,6 @@ static int catia_unlinkat(vfs_handle_struct *handle,
 	struct catia_cache *cc = NULL;
 	struct smb_filename *smb_fname_tmp = NULL;
 	char *name = NULL;
-	NTSTATUS status;
 	int ret;
 
 	ret = CATIA_FETCH_FSP_PRE_NEXT(talloc_tos(), handle, dirfsp, &cc);
@@ -673,11 +678,12 @@ static int catia_unlinkat(vfs_handle_struct *handle,
 		return ret;
 	}
 
-	status = catia_string_replace_allocate(handle->conn,
-					smb_fname->base_name,
-					&name, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		goto out;
 	}
 
@@ -709,17 +715,16 @@ static int catia_lchown(vfs_handle_struct *handle,
 			gid_t gid)
 {
 	char *name = NULL;
-	NTSTATUS status;
 	int ret;
 	int saved_errno;
 	struct smb_filename *catia_smb_fname = NULL;
 
-	status = catia_string_replace_allocate(handle->conn,
-					smb_fname->base_name,
-					&name,
-					vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 	catia_smb_fname = synthetic_smb_fname(talloc_tos(),
@@ -748,16 +753,15 @@ static int catia_mkdirat(vfs_handle_struct *handle,
 			mode_t mode)
 {
 	char *name = NULL;
-	NTSTATUS status;
 	int ret;
 	struct smb_filename *catia_smb_fname = NULL;
 
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname->base_name,
-				&name,
-				vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 	catia_smb_fname = synthetic_smb_fname(talloc_tos(),
@@ -787,15 +791,14 @@ static int catia_chdir(vfs_handle_struct *handle,
 {
 	char *name = NULL;
 	struct smb_filename *catia_smb_fname = NULL;
-	NTSTATUS status;
 	int ret;
 
-	status = catia_string_replace_allocate(handle->conn,
-					smb_fname->base_name,
-					&name,
-					vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
@@ -844,13 +847,14 @@ catia_realpath(vfs_handle_struct *handle,
 	char *mapped_name = NULL;
 	struct smb_filename *catia_smb_fname = NULL;
 	struct smb_filename *return_fname = NULL;
-	NTSTATUS status;
+	int ret;
 
-	status = catia_string_replace_allocate(handle->conn,
-					smb_fname->base_name,
-				        &mapped_name, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &mapped_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return NULL;
 	}
 
@@ -885,17 +889,18 @@ catia_fstreaminfo(struct vfs_handle_struct *handle,
 	struct smb_filename *smb_fname = NULL;
 	unsigned int num_streams = 0;
 	struct stream_struct *streams = NULL;
+	int ret;
 
 	smb_fname = fsp->fsp_name;
 	*_num_streams = 0;
 	*_streams = NULL;
 
-	status = catia_string_replace_allocate(handle->conn,
-				smb_fname->base_name,
-				&mapped_name,
-				vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		return status;
+	ret = catia_string_replace_allocate(handle->conn,
+					    smb_fname->base_name,
+					    &mapped_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		return map_nt_error_from_unix(ret);
 	}
 
 	status = synthetic_pathref(talloc_tos(),
@@ -940,13 +945,13 @@ catia_fstreaminfo(struct vfs_handle_struct *handle,
 			stream_type += 1;
 		}
 
-		status = catia_string_replace_allocate(handle->conn,
-						stream_name,
-						&mapped_name,
-						vfs_translate_to_windows);
-		if (!NT_STATUS_IS_OK(status)) {
+		ret = catia_string_replace_allocate(handle->conn,
+						    stream_name,
+						    &mapped_name,
+						    vfs_translate_to_windows);
+		if (ret != 0) {
 			TALLOC_FREE(streams);
-			return status;
+			return map_nt_error_from_unix(ret);
 		}
 
 		if (stream_type != NULL) {
@@ -1078,14 +1083,15 @@ static ssize_t catia_fgetxattr(struct vfs_handle_struct *handle,
 			       size_t size)
 {
 	char *mapped_xattr_name = NULL;
-	NTSTATUS status;
 	ssize_t result;
+	int ret;
 
-	status = catia_string_replace_allocate(handle->conn,
-					       name, &mapped_xattr_name,
-					       vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    name,
+					    &mapped_xattr_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
@@ -1123,13 +1129,14 @@ static int catia_fremovexattr(struct vfs_handle_struct *handle,
 			      const char *name)
 {
 	char *mapped_name = NULL;
-	NTSTATUS status;
 	int ret;
 
-	status = catia_string_replace_allocate(handle->conn,
-				name, &mapped_name, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    name,
+					    &mapped_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
@@ -1148,13 +1155,14 @@ static int catia_fsetxattr(struct vfs_handle_struct *handle,
 			   int flags)
 {
 	char *mapped_xattr_name = NULL;
-	NTSTATUS status;
 	int ret;
 
-	status = catia_string_replace_allocate(
-		handle->conn, name, &mapped_xattr_name, vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	ret = catia_string_replace_allocate(handle->conn,
+					    name,
+					    &mapped_xattr_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		errno = ret;
 		return -1;
 	}
 
@@ -1611,10 +1619,10 @@ static bool catia_lock(vfs_handle_struct *handle,
 	return ok;
 }
 
-static int catia_kernel_flock(struct vfs_handle_struct *handle,
-			      struct files_struct *fsp,
-			      uint32_t share_access,
-			      uint32_t access_mask)
+static int catia_filesystem_sharemode(struct vfs_handle_struct *handle,
+				      struct files_struct *fsp,
+				      uint32_t share_access,
+				      uint32_t access_mask)
 {
 	struct catia_cache *cc = NULL;
 	int ret;
@@ -1624,7 +1632,10 @@ static int catia_kernel_flock(struct vfs_handle_struct *handle,
 		return -1;
 	}
 
-	ret = SMB_VFS_NEXT_KERNEL_FLOCK(handle, fsp, share_access, access_mask);
+	ret = SMB_VFS_NEXT_FILESYSTEM_SHAREMODE(handle,
+						fsp,
+						share_access,
+						access_mask);
 
 	CATIA_FETCH_FSP_POST_NEXT(&cc, fsp);
 
@@ -1785,14 +1796,14 @@ static NTSTATUS catia_create_dfs_pathat(struct vfs_handle_struct *handle,
 	const char *path = smb_fname->base_name;
 	struct smb_filename *mapped_smb_fname = NULL;
 	NTSTATUS status;
+	int ret;
 
-	status = catia_string_replace_allocate(handle->conn,
-					path,
-					&mapped_name,
-					vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
-		return status;
+	ret = catia_string_replace_allocate(handle->conn,
+					    path,
+					    &mapped_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		return map_nt_error_from_unix(ret);
 	}
 	mapped_smb_fname = synthetic_smb_fname(talloc_tos(),
 					mapped_name,
@@ -1826,14 +1837,14 @@ static NTSTATUS catia_read_dfs_pathat(struct vfs_handle_struct *handle,
 	const char *path = smb_fname->base_name;
 	struct smb_filename *mapped_smb_fname = NULL;
 	NTSTATUS status;
+	int ret;
 
-	status = catia_string_replace_allocate(handle->conn,
-					path,
-					&mapped_name,
-					vfs_translate_to_unix);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
-		return status;
+	ret = catia_string_replace_allocate(handle->conn,
+					    path,
+					    &mapped_name,
+					    vfs_translate_to_unix);
+	if (ret != 0) {
+		return map_nt_error_from_unix(ret);
 	}
 	mapped_smb_fname = synthetic_smb_fname(talloc_tos(),
 					mapped_name,
@@ -1892,7 +1903,7 @@ static struct vfs_fn_pointers vfs_catia_fns = {
 	.ftruncate_fn = catia_ftruncate,
 	.fallocate_fn = catia_fallocate,
 	.lock_fn = catia_lock,
-	.kernel_flock_fn = catia_kernel_flock,
+	.filesystem_sharemode_fn = catia_filesystem_sharemode,
 	.linux_setlease_fn = catia_linux_setlease,
 	.getlock_fn = catia_getlock,
 	.realpath_fn = catia_realpath,
