@@ -60,7 +60,7 @@ struct winbindd_cli_state {
 			    struct winbindd_response *presp);
 	struct winbindd_request *request;         /* Request from client */
 	struct tevent_queue *out_queue;
-	struct winbindd_response *response;        /* Respose to client */
+	struct winbindd_response *response;        /* Response to client */
 	struct tevent_req *io_req; /* wb_req_read_* or wb_resp_write_* */
 
 	struct getpwent_state *pwent_state; /* State for getpwent() */
@@ -130,7 +130,6 @@ struct winbindd_domain {
 	uint32_t domain_trust_attribs;           /* Trust attribs from netlogon.h */
 	struct winbindd_domain *routing_domain;
 	bool initialized;		       /* Did we already ask for the domain mode? */
-	bool native_mode;                      /* is this a win2k domain in native mode ? */
 	bool active_directory;                 /* is this a win2k active directory ? */
 	bool primary;                          /* is this our primary domain ? */
 	bool internal;                         /* BUILTIN and member SAM */
@@ -279,6 +278,14 @@ struct winbindd_methods {
 				    struct dom_sid **sid_mem, char ***names,
 				    uint32_t **name_types);
 
+	/* find all members of the alias with the specified alias_sid */
+	NTSTATUS (*lookup_aliasmem)(struct winbindd_domain *domain,
+				    TALLOC_CTX *mem_ctx,
+				    const struct dom_sid *alias_sid,
+				    enum lsa_SidType type,
+				    uint32_t *num_sids,
+				    struct dom_sid **sid_mem);
+
 	/* return the lockout policy */
 	NTSTATUS (*lockout_policy)(struct winbindd_domain *domain,
  				   TALLOC_CTX *mem_ctx,
